@@ -53,9 +53,7 @@ class TestImporterIterModules(unittest.TestCase):
             collector.add_in_memory(r)
 
         f = OxidizedFinder()
-        f.add_resources(
-            collector.oxidize(python_exe=os.environ.get("PYTHON_SYS_EXECUTABLE"))[0]
-        )
+        f.add_resources(collector.oxidize()[0])
 
         return f
 
@@ -110,6 +108,7 @@ class TestImporterIterModules(unittest.TestCase):
 
     def test_iter_modules_nested(self):
         self._make_package("a.b")
+        self._make_package("a.b.c")
         (self.td / "one.py").touch()
 
         f = self._finder_from_td()
@@ -133,7 +132,7 @@ class TestImporterIterModules(unittest.TestCase):
         f = self._finder_from_td()
 
         name = "on"
-        path = pathlib.Path(sys.executable, name)
+        path = os.path.join(f.path_hook_base_str, name)
         path_entry_finder = f.path_hook(path)
 
         with self.subTest(prefix="", module_iterator="OxidizedPathEntryFinder"):
@@ -152,12 +151,14 @@ class TestImporterIterModules(unittest.TestCase):
                 for mi in res:
                     self.assertIsInstance(mi.module_finder, type(path_entry_finder))
                     self.assertEqual(
-                        mi.module_finder._package, path_entry_finder._package)
+                        mi.module_finder._package, path_entry_finder._package
+                    )
 
         sys.path = [sys.executable]
         sys.meta_path = [f]
         with patch.dict(sys.modules):
             import on
+
             self.assertEqual(on.__name__, name)
             self.assertEqual(on.__path__, [str(path)])
             with patch.object(sys, "path_hooks", [f.path_hook]):
